'use strict';
const path = require('path');
const fs = require('fs');
const readdir = require('recursive-readdir');
const mkdirp = require('mkdirp');
const fullTemplate = [
    '<?xml version="1.0" encoding="iso-8859-1"?>',
    '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">',
    '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"$attrs>$icons</svg>'
].join('');
function readdirPromise(dir, ignore) {
    return new Promise((resolve, reject) => {
        readdir(dir, ignore, (err, files) => {
            if (err) {
                reject(err);
            }
            resolve(files);
        });
    });
}
function readFilePromise(filepath, encode) {
    return new Promise((resolve, reject) => {
        fs.readFile(filepath, encode, (err, files) => {
            if (err) {
                reject(err);
            }
            resolve(files);
        });
    });
}
function writeFilePromise(filepath, data) {
    return new Promise((resolve, reject) => {
        mkdirp(path.dirname(filepath), (mkdirpError) => {
            if (mkdirpError) {
                reject(mkdirpError);
            }
            fs.writeFile(filepath, data, (writeError, files) => {
                if (writeError) {
                    reject(writeError);
                }
                resolve(files);
            });
        });
    });
}
function makeSprite(dir, options) {
    if (!dir) {
        throw new Error('`dir` required');
    }
    let template = fullTemplate;
    options = Object.assign({
        ignore: [],
        parentAttrs: {},
        inline: false,
        iconAttrs: {},
        iconPrefix: '',
        iconSuffix: ''
    }, options);
    options.ignore.push('!*.svg');
    if (options.inline) {
        options.parentAttrs = Object.assign({
            width: 0,
            height: 0,
            style: 'position:absolute'
        }, options.parentAttrs);
        template = `<svg$attrs>$icons</svg>`;
    }
    const parentAttrs = makeAttributes(options.parentAttrs);
    template = template.replace('$attrs', parentAttrs);
    return readdirPromise(dir, options.ignore)
        .then((files) => Promise.all(files.map((filepath) => {
        const name = path.basename(filepath);
        return readFilePromise(filepath, 'utf-8').then((content) => ({ name: name, content: content }));
    })))
        .then((files) => {
        const icons = files.map((file) => {
            file.content = stripExtraLines(file.content);
            file.content = updateAttributes(file, options);
            file.content = file.content.replace(/\<svg/g, '<symbol').replace(/\<\/svg>/g, '</symbol>');
            return file.content;
        });
        template = template.replace('$icons', icons.join(''));
        return {
            sprite: template,
            write: (filepath) => writeFilePromise(filepath, template)
        };
    });
}
exports.makeSprite = makeSprite;
function makeAttributes(attrs) {
    let str = '';
    Object.keys(attrs).forEach((property) => {
        str += ` ${property}="${attrs[property]}"`;
    });
    return str;
}
function updateAttributes(file, options) {
    const attrs = options.iconAttrs;
    const name = options.iconPrefix + path.basename(file.name, '.svg') + options.iconSuffix;
    attrs.id = name;
    Object.keys(attrs).forEach((property) => {
        const regexp = new RegExp(` ${property}="([^"]+)"`, 'g');
        if (regexp.test(file.content)) {
            file.content = file.content.replace(regexp, ` ${property}="${attrs[property]}"`);
        }
        else {
            file.content = file.content.replace(/\<svg/g, `<svg ${property}="${attrs[property]}"`);
        }
    });
    return file.content;
}
function stripExtraLines(content) {
    return content
        .replace(/\r?\n|\r|\t/g, '')
        .replace(/\<(\?xml|(\!DOCTYPE[^\>\[]+(\[[^\]]+)?))+[^>]+\>/g, '')
        .replace(/\sxmlns="http:\/\/www.w3.org\/2000\/svg"/g, '');
}
